import { createTransformStream } from "@datastream/core";
import { compile } from "ajv-cmd";
const ajvDefaults = {
  strict: true,
  coerceTypes: true,
  allErrors: true,
  useDefaults: "empty",
  messages: true
};
const transpileSchema = (schema, ajvOptions) => {
  const options = { ...ajvDefaults, ...ajvOptions };
  return compile(schema, options);
};
const validateStream = ({ schema, idxStart, resultKey, ...ajvOptions }, streamOptions) => {
  idxStart ??= 0;
  if (typeof schema !== "function") {
    schema = transpileSchema(schema);
  }
  const value = {};
  let idx = idxStart - 1;
  const transform = (chunk, enqueue) => {
    idx += 1;
    const chunkValid = schema(chunk);
    if (!chunkValid) {
      for (const error of schema.errors) {
        const { id, keys, message } = processError(error);
        if (!value[id]) {
          value[id] = { id, keys, message, idx: [] };
        }
        value[id].idx.push(idx);
      }
    }
    enqueue(chunk);
  };
  const stream = createTransformStream(transform, streamOptions);
  stream.result = () => ({ key: resultKey ?? "validate", value });
  return stream;
};
const processError = (error) => {
  const message = error.message || "";
  let id = error.schemaPath;
  let keys = [];
  if (error.keyword === "errorMessage") {
    error.params.errors.forEach((error2) => {
      const value = makeKeys(error2);
      if (value)
        keys.push(value);
    });
    keys = [...new Set(keys.sort())];
  } else {
    keys.push(makeKeys(error));
  }
  if (!error.instancePath && keys.length) {
    id += `/${keys.join("|")}`;
  }
  return { id, keys, message };
};
const makeKeys = (error) => {
  return error.params.missingProperty || error.params.additionalProperty || error.instancePath.replace("/", "");
};
var validate_default = validateStream;
export {
  validate_default as default,
  transpileSchema,
  validateStream
};
